@@ -5,13 +5,14 @@ from datajoint.user_tables import UserTable
 from datajoint import VirtualModule
 import datetime
 import numpy as np
+import re
 from .error import InvalidRestriction, UnsupportedTableType
 
 DAY = 24 * 60 * 60
 DEFAULT_FETCH_LIMIT = 1000  # Stop gap measure to deal with super large tables
 
 
-class _DJConnector():
+class _DJConnector:
     """Primary connector that communicates with a DataJoint database server."""
 
     @staticmethod
@@ -26,9 +27,9 @@ class _DJConnector():
         :param password: Password of user
         :type password: str
         """
-        dj.config['database.host'] = database_address
-        dj.config['database.user'] = username
-        dj.config['database.password'] = password
+        dj.config["database.host"] = database_address
+        dj.config["database.user"] = username
+        dj.config["database.password"] = password
 
         # Attempt to connect return true if successful, false is failed
         dj.conn(reset=True)
@@ -48,11 +49,18 @@ class _DJConnector():
         _DJConnector._set_datajoint_config(jwt_payload)
 
         # Attempt to connect return true if successful, false is failed
-        return [row[0] for row in dj.conn().query("""
-        SELECT SCHEMA_NAME FROM information_schema.schemata
-        WHERE SCHEMA_NAME NOT IN ("information_schema", "sys", "performance_schema", "mysql")
-        ORDER BY SCHEMA_NAME
-        """)]
+        return [
+            row[0]
+            for row in dj.conn().query(
+                """
+                SELECT SCHEMA_NAME FROM information_schema.schemata
+                WHERE SCHEMA_NAME NOT IN (
+                    "information_schema", "sys", "performance_schema", "mysql"
+                )
+                ORDER BY SCHEMA_NAME
+                """
+            )
+        ]
 
     @staticmethod
     def _list_tables(jwt_payload: dict, schema_name: str) -> dict:
@@ -73,34 +81,42 @@ class _DJConnector():
         # Get list of tables names
         tables_name = dj.Schema(schema_name, create_schema=False).list_tables()
         # Dict to store list of table name for each type
-        tables_dict_list = dict(manual=[], lookup=[], computed=[],
-                                imported=[], part=[])
+        tables_dict_list = dict(manual=[], lookup=[], computed=[], imported=[], part=[])
         # Loop through each table name to figure out what type it is and add them to
         # tables_dict_list
         for table_name in tables_name:
             table_type = dj.diagram._get_tier(
-                '`' + schema_name + '`.`' + table_name + '`').__name__
-            if table_type == 'Manual':
-                tables_dict_list['manual'].append(dj.utils.to_camel_case(table_name))
-            elif table_type == 'Lookup':
-                tables_dict_list['lookup'].append(dj.utils.to_camel_case(table_name))
-            elif table_type == 'Computed':
-                tables_dict_list['computed'].append(dj.utils.to_camel_case(table_name))
-            elif table_type == 'Imported':
-                tables_dict_list['imported'].append(dj.utils.to_camel_case(table_name))
-            elif table_type == 'Part':
-                table_name_parts = table_name.split('__')
-                tables_dict_list['part'].append(
-                    to_camel_case(table_name_parts[-2]) + '.' +
-                    to_camel_case(table_name_parts[-1]))
+                "`" + schema_name + "`.`" + table_name + "`"
+            ).__name__
+            if table_type == "Manual":
+                tables_dict_list["manual"].append(dj.utils.to_camel_case(table_name))
+            elif table_type == "Lookup":
+                tables_dict_list["lookup"].append(dj.utils.to_camel_case(table_name))
+            elif table_type == "Computed":
+                tables_dict_list["computed"].append(dj.utils.to_camel_case(table_name))
+            elif table_type == "Imported":
+                tables_dict_list["imported"].append(dj.utils.to_camel_case(table_name))
+            elif table_type == "Part":
+                table_name_parts = table_name.split("__")
+                tables_dict_list["part"].append(
+                    to_camel_case(table_name_parts[-2])
+                    + "."
+                    + to_camel_case(table_name_parts[-1])
+                )
             else:
-                raise UnsupportedTableType(table_name + ' is of unknown table type')
+                raise UnsupportedTableType(table_name + " is of unknown table type")
         return tables_dict_list
 
     @staticmethod
-    def _fetch_records(query,
-                       restriction: list = [], limit: int = 1000, page: int = 1,
-                       order=['KEY ASC']) -> tuple:
+    def _fetch_records(
+        query,
+        restriction: list = [],
+        limit: int = 1000,
+        page: int = 1,
+        order=["KEY ASC"],
+        fetch_blobs=False,
+        fetch_args=[],
+    ) -> tuple:
         """
         Get records from query.
 
@@ -125,12 +141,29 @@ class _DJConnector():
         attributes = query.heading.attributes
         # Fetch tuples without blobs as dict to be used to create a
         #   list of tuples for returning
-        query_restricted = query & dj.AndList([
-            _DJConnector._filter_to_restriction(f, attributes[f['attributeName']].type)
-            for f in restriction])
-        non_blobs_rows = query_restricted.fetch(*query.heading.non_blobs, as_dict=True,
-                                                limit=limit, offset=(page-1)*limit,
-                                                order_by=order)
+        query_restricted = query & dj.AndList(
+            [
+                _DJConnector._filter_to_restriction(
+                    f, attributes[f["attributeName"]].type
+                )
+                for f in restriction
+            ]
+        )
+
+        if fetch_blobs and not fetch_args:
+            fetch_args = [*query.heading.attributes]
+        elif not fetch_args:
+            fetch_args = query.heading.non_blobs
+        else:
+            attributes = {k: v for k, v in attributes.items() if k in fetch_args}
+
+        non_blobs_rows = query_restricted.fetch(
+            *fetch_args,
+            as_dict=True,
+            limit=limit,
+            offset=(page - 1) * limit,
+            order_by=order,
+        )
 
         # Buffer list to be return
         rows = []
@@ -147,17 +180,24 @@ class _DJConnector():
                     if non_blobs_row[attribute_name] is None:
                         # If it is none then just append None
                         row.append(None)
-                    elif attribute_info.type == 'date':
+                    elif attribute_info.type == "date":
                         # Date attribute type covert to epoch time
-                        row.append((non_blobs_row[attribute_name] -
-                                    datetime.date(1970, 1, 1)).days * DAY)
-                    elif attribute_info.type == 'time':
+                        row.append(
+                            (
+                                non_blobs_row[attribute_name]
+                                - datetime.date(1970, 1, 1)
+                            ).days
+                            * DAY
+                        )
+                    elif attribute_info.type == "time":
                         # Time attirbute, return total seconds
                         row.append(non_blobs_row[attribute_name].total_seconds())
-                    elif attribute_info.type in ('datetime', 'timestamp'):
+                    elif re.match(r"^datetime.*$", attribute_info.type) or re.match(
+                        r"timestamp", attribute_info.type
+                    ):
                         # Datetime or timestamp, use timestamp to covert to epoch time
                         row.append(non_blobs_row[attribute_name].timestamp())
-                    elif attribute_info.type[0:7] == 'decimal':
+                    elif attribute_info.type[0:7] == "decimal":
                         # Covert decimal to string
                         row.append(str(non_blobs_row[attribute_name]))
                     else:
@@ -169,8 +209,11 @@ class _DJConnector():
                             row.append(non_blobs_row[attribute_name])
                 else:
                     # Attribute is blob type thus fill it in string instead
-                    row.append('=BLOB=')
-
+                    (
+                        row.append(non_blobs_row[attribute_name])
+                        if fetch_blobs
+                        else row.append("=BLOB=")
+                    )
             # Add the row list to tuples
             rows.append(row)
         return list(attributes.keys()), rows, len(query_restricted)
@@ -192,28 +235,35 @@ class _DJConnector():
         query_attributes = dict(primary=[], secondary=[])
         for attribute_name, attribute_info in query.heading.attributes.items():
             if attribute_info.in_key:
-                query_attributes['primary'].append((
-                    attribute_name,
-                    attribute_info.type,
-                    attribute_info.nullable,
-                    attribute_info.default,
-                    attribute_info.autoincrement
-                    ))
+                query_attributes["primary"].append(
+                    (
+                        attribute_name,
+                        attribute_info.type,
+                        attribute_info.nullable,
+                        attribute_info.default,
+                        attribute_info.autoincrement,
+                    )
+                )
             else:
-                query_attributes['secondary'].append((
-                    attribute_name,
-                    attribute_info.type,
-                    attribute_info.nullable,
-                    attribute_info.default,
-                    attribute_info.autoincrement
-                    ))
+                query_attributes["secondary"].append(
+                    (
+                        attribute_name,
+                        attribute_info.type,
+                        attribute_info.nullable,
+                        attribute_info.default,
+                        attribute_info.autoincrement,
+                    )
+                )
 
-        return dict(attribute_headers=['name', 'type', 'nullable',
-                                       'default', 'autoincrement'],
-                    attributes=query_attributes)
+        return dict(
+            attribute_headers=["name", "type", "nullable", "default", "autoincrement"],
+            attributes=query_attributes,
+        )
 
     @staticmethod
-    def _get_table_definition(jwt_payload: dict, schema_name: str, table_name: str) -> str:
+    def _get_table_definition(
+        jwt_payload: dict, schema_name: str, table_name: str
+    ) -> str:
         """
         Get the table definition.
 
@@ -234,8 +284,9 @@ class _DJConnector():
         return getattr(local_values[schema_name], table_name).describe()
 
     @staticmethod
-    def _insert_tuple(jwt_payload: dict, schema_name: str, table_name: str,
-                      tuple_to_insert: dict):
+    def _insert_tuple(
+        jwt_payload: dict, schema_name: str, table_name: str, tuple_to_insert: dict
+    ):
         """
         Insert record as tuple into table.
 
@@ -255,8 +306,9 @@ class _DJConnector():
         getattr(schema_virtual_module, table_name).insert(tuple_to_insert)
 
     @staticmethod
-    def _record_dependency(jwt_payload: dict, schema_name: str, table_name: str,
-                           restriction: list = []) -> list:
+    def _record_dependency(
+        jwt_payload: dict, schema_name: str, table_name: str, restriction: list = []
+    ) -> list:
         """
         Return summary of dependencies associated with a restricted table. Will only show
         dependencies that user has access to.
@@ -279,19 +331,35 @@ class _DJConnector():
         table = getattr(virtual_module, table_name)
         attributes = table.heading.attributes
         # Retrieve dependencies of related to retricted
-        dependencies = [dict(schema=descendant.database, table=descendant.table_name,
-                             accessible=True, count=len(
-                                (table if descendant.full_table_name == table.full_table_name
-                                 else descendant * table) & dj.AndList([
-                                    _DJConnector._filter_to_restriction(
-                                        f, attributes[f['attributeName']].type)
-                                    for f in restriction])))
-                        for descendant in table().descendants(as_objects=True)]
+        dependencies = [
+            dict(
+                schema=descendant.database,
+                table=descendant.table_name,
+                accessible=True,
+                count=len(
+                    (
+                        table
+                        if descendant.full_table_name == table.full_table_name
+                        else descendant * table
+                    )
+                    & dj.AndList(
+                        [
+                            _DJConnector._filter_to_restriction(
+                                f, attributes[f["attributeName"]].type
+                            )
+                            for f in restriction
+                        ]
+                    )
+                ),
+            )
+            for descendant in table().descendants(as_objects=True)
+        ]
         return dependencies
 
     @staticmethod
-    def _update_tuple(jwt_payload: dict, schema_name: str, table_name: str,
-                      tuple_to_update: dict):
+    def _update_tuple(
+        jwt_payload: dict, schema_name: str, table_name: str, tuple_to_update: dict
+    ):
         """
         Update record as tuple into table.
 
@@ -309,11 +377,19 @@ class _DJConnector():
 
         schema_virtual_module = dj.VirtualModule(schema_name, schema_name)
         with conn.transaction:
-            [getattr(schema_virtual_module, table_name).update1(t) for t in tuple_to_update]
+            [
+                getattr(schema_virtual_module, table_name).update1(t)
+                for t in tuple_to_update
+            ]
 
     @staticmethod
-    def _delete_records(jwt_payload: dict, schema_name: str, table_name: str,
-                        restriction: list = [], cascade: bool = False):
+    def _delete_records(
+        jwt_payload: dict,
+        schema_name: str,
+        table_name: str,
+        restriction: list = [],
+        cascade: bool = False,
+    ):
         """
         Delete a specific record based on the restriction given.
 
@@ -338,20 +414,23 @@ class _DJConnector():
         table = _DJConnector._get_table_object(schema_virtual_module, table_name)
         attributes = table.heading.attributes
         restrictions = [
-            _DJConnector._filter_to_restriction(f, attributes[f['attributeName']].type)
-            for f in restriction]
+            _DJConnector._filter_to_restriction(f, attributes[f["attributeName"]].type)
+            for f in restriction
+        ]
 
         # Compute restriction
         query = table & dj.AndList(restrictions)
         # Check if there is only 1 tuple to delete otherwise raise error
         if len(query) == 0:
-            raise InvalidRestriction('Nothing to delete')
+            raise InvalidRestriction("Nothing to delete")
 
         # All check pass thus proceed to delete
         query.delete(safemode=False) if cascade else query.delete_quick()
 
     @staticmethod
-    def _get_table_object(schema_virtual_module: VirtualModule, table_name: str) -> UserTable:
+    def _get_table_object(
+        schema_virtual_module: VirtualModule, table_name: str
+    ) -> UserTable:
         """
         Helper method for getting the table object based on the table name provided.
 
@@ -363,10 +442,11 @@ class _DJConnector():
         :rtype: :class:`~datajoint.user_tables.UserTable`
         """
         # Split the table name by '.' for dealing with part tables
-        table_name_parts = table_name.split('.')
+        table_name_parts = table_name.split(".")
         if len(table_name_parts) == 2:
-            return getattr(getattr(schema_virtual_module, table_name_parts[0]),
-                           table_name_parts[1])
+            return getattr(
+                getattr(schema_virtual_module, table_name_parts[0]), table_name_parts[1]
+            )
         else:
             return getattr(schema_virtual_module, table_name_parts[0])
 
@@ -383,21 +463,28 @@ class _DJConnector():
         :return: DataJoint-compatible restriction
         :rtype: str
         """
-        if attribute_filter['operation'] in ('>', '<', '>=', '<='):
-            operation = attribute_filter['operation']
-        elif attribute_filter['value'] is None:
-            operation = (' IS ' if attribute_filter['operation'] == '='
-                         else ' IS NOT ')
+        if attribute_filter["operation"] in (">", "<", ">=", "<="):
+            operation = attribute_filter["operation"]
+        elif attribute_filter["value"] is None:
+            operation = " IS " if attribute_filter["operation"] == "=" else " IS NOT "
         else:
-            operation = attribute_filter['operation']
+            operation = attribute_filter["operation"]
 
-        if (isinstance(attribute_filter['value'], str) and
-                not attribute_filter['value'].isnumeric()):
-            value = (f"X'{attribute_filter['value'].replace('-', '')}'"
-                     if attribute_type == 'uuid' else f"'{attribute_filter['value']}'")
+        if (
+            isinstance(attribute_filter["value"], str)
+            and not attribute_filter["value"].isnumeric()
+        ):
+            value = (
+                f"X'{attribute_filter['value'].replace('-', '')}'"
+                if attribute_type == "uuid"
+                else f"'{attribute_filter['value']}'"
+            )
         else:
-            value = ('NULL' if attribute_filter['value'] is None
-                     else attribute_filter['value'])
+            value = (
+                "NULL"
+                if attribute_filter["value"] is None
+                else attribute_filter["value"]
+            )
         return f"{attribute_filter['attributeName']}{operation}{value}"
 
     @staticmethod
@@ -411,7 +498,7 @@ class _DJConnector():
         :return: DataJoint connection object.
         :rtype: :class:`~datajoint.connection.Connection`
         """
-        dj.config['database.host'] = jwt_payload['databaseAddress']
-        dj.config['database.user'] = jwt_payload['username']
-        dj.config['database.password'] = jwt_payload['password']
+        dj.config["database.host"] = jwt_payload["databaseAddress"]
+        dj.config["database.user"] = jwt_payload["username"]
+        dj.config["database.password"] = jwt_payload["password"]
         return dj.conn(reset=True)
